@@ -37,6 +37,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include "config.h"
 #include "mega-http-client.h"
 #include "mega-http-io-stream.h"
 
@@ -293,6 +294,14 @@ static gboolean do_receive_headers(MegaHttpClient* http_client, GCancellable* ca
         goto err;
       }
 
+      if (status == 500 && g_str_equal(message, "Server Too Busy"))
+      {
+        g_set_error(err, MEGA_HTTP_CLIENT_ERROR, MEGA_HTTP_CLIENT_ERROR_SERVER_BUSY, "Server returned status %d: %s", status, message);
+        g_free(header);
+        g_free(message);
+        goto err;
+      }
+
       if (status != 200 && status != 201)
       {
         g_set_error(err, MEGA_HTTP_CLIENT_ERROR, MEGA_HTTP_CLIENT_ERROR_OTHER, "Server returned status %d: %s", status, message);
@@ -726,9 +735,9 @@ GString* mega_http_client_post_simple(MegaHttpClient* http_client, const gchar*
     return NULL;
   }
 
-  if (response_length > 32 * MB) 
+  if (response_length > 256 * MB) 
   {
-    g_set_error(err, MEGA_HTTP_CLIENT_ERROR, MEGA_HTTP_CLIENT_ERROR_OTHER, "Response length over 32 MiB not supported (for post_simple): %s", url);
+    g_set_error(err, MEGA_HTTP_CLIENT_ERROR, MEGA_HTTP_CLIENT_ERROR_OTHER, "Response length over 256 MiB not supported (for post_simple): %s", url);
     g_object_unref(io);
     return NULL;
   }
@@ -837,6 +846,9 @@ gssize mega_http_client_read(MegaHttpClient* http_client, guchar* buffer, gsize
 
   MegaHttpClientPrivate* priv = http_client->priv;
 
+  if (priv->conn_state == CONN_STATE_NONE || priv->conn_state == CONN_STATE_NONE_CONNECTED)
+    return 0;
+
   if (!goto_state(http_client, CONN_STATE_HEADERS_RECEIVED, cancellable, &local_err))
   {
     g_propagate_error(err, local_err);
@@ -1003,6 +1015,7 @@ static void mega_http_client_init(MegaHttpClient *http_client)
 
   priv->client = g_socket_client_new();
   g_socket_client_set_timeout(priv->client, 60);
+  g_socket_client_set_family(priv->client, G_SOCKET_FAMILY_IPV4);
   priv->request_headers = g_hash_table_new_full(stri_hash, stri_equal, g_free, g_free);
   priv->response_headers = g_hash_table_new_full(stri_hash, stri_equal, g_free, g_free);
   priv->regex_url = g_regex_new("^([a-z]+)://([a-z0-9.-]+(?::([0-9]+))?)(/.+)?$", G_REGEX_CASELESS, 0, NULL);
@@ -1010,7 +1023,7 @@ static void mega_http_client_init(MegaHttpClient *http_client)
 
   // set default headers
   mega_http_client_set_header(http_client, "Referer", "https://mega.co.nz/");
-  mega_http_client_set_header(http_client, "User-Agent", "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.70 Safari/537.17");
+  mega_http_client_set_header(http_client, "User-Agent", "Megatools (" VERSION ")");
   mega_http_client_set_header(http_client, "Connection", "keep-alive");
 }
 
